@@ -428,6 +428,12 @@ common_opts = [
     cfg.StrOpt('injected_config_location', default='/etc/trove/conf.d',
                help='Path to folder on the Guest where config files will be '
                     'injected during instance creation.'),
+    cfg.StrOpt('injected_config_owner', default='ubuntu',
+               help='Owner of the Guest Agent directory and files to be '
+                    'injected during instance creation.'),
+    cfg.StrOpt('injected_config_group', default='root',
+               help='Group of the Guest Agent directory and files to be '
+                    'injected during instance creation.'),
     cfg.StrOpt('guest_config',
                default='/etc/trove/trove-guestagent.conf',
                help='Path to the Guest Agent config file to be injected '
@@ -441,6 +447,10 @@ common_opts = [
                 help='Extension for default datastore managers. '
                      'Allows the use of custom managers for each of '
                      'the datastores supported by Trove.'),
+    cfg.StrOpt('docker_bridge_network_ip', default=None,
+               help='Docker bridge network IP.'),
+    cfg.ListOpt('docker_insecure_registries', default=[],
+                help='Docker insecure registries for image development.'),
     cfg.StrOpt('template_path', default='/etc/trove/templates/',
                help='Path which leads to datastore templates.'),
     cfg.BoolOpt('sql_query_logging', default=False,
@@ -1526,7 +1536,8 @@ guest_agent_opts = [
     ),
     cfg.StrOpt(
         'container_registry_password',
-        help='The plaintext registry password.'
+        help='The plaintext registry password.',
+        secret=True,
     ),
 ]
 
