@@ -26,11 +26,10 @@ from troveclient.compat import exceptions
 from trove.common.utils import generate_uuid
 from trove.common.utils import poll_until
 from trove import tests
-from trove.tests.api import configurations
 from trove.tests.api.instances import CheckInstance
 from trove.tests.api.instances import instance_info
-from trove.tests.api.instances import TIMEOUT_INSTANCE_CREATE
 from trove.tests.api.instances import TIMEOUT_INSTANCE_DELETE
+from trove.tests.api.instances import TIMEOUT_INSTANCE_RESTORE
 from trove.tests.config import CONFIG
 from trove.tests.scenario import runners
 from trove.tests.scenario.runners.test_runners import SkipKnownBug
@@ -44,20 +43,22 @@ class SlaveInstanceTestInfo(object):
         self.replicated_db = generate_uuid()
 
 
-REPLICATION_GROUP = "dbaas.api.replication"
 slave_instance = SlaveInstanceTestInfo()
 existing_db_on_master = generate_uuid()
 backup_count = None
 
 
 def _get_user_count(server_info):
-    cmd = ('mysql -BNq -e \\\'select count\\(*\\) from mysql.user'
-           ' where user like \\\"slave_%\\\"\\\'')
+    cmd = (
+        'docker exec -e MYSQL_PWD=$(sudo cat /var/lib/mysql/conf.d/root.cnf | '
+        'grep password | awk "{print \$3}") database mysql -uroot -N -e '
+        '"select count(*) from mysql.user where user like \\"slave_%\\""'
+    )
     server = create_server_connection(server_info.id)
 
     try:
         stdout = server.execute(cmd)
-        return int(stdout)
+        return int(stdout.rstrip())
     except Exception as e:
         fail("Failed to execute command: %s, error: %s" % (cmd, str(e)))
 
@@ -65,8 +66,13 @@ def _get_user_count(server_info):
 def slave_is_running(running=True):
     def check_slave_is_running():
         server = create_server_connection(slave_instance.id)
-        cmd = ("mysqladmin extended-status "
-               "| awk '/Slave_running/{print $4}'")
+        cmd = (
+            'docker exec -e MYSQL_PWD=$(sudo cat '
+            '/var/lib/mysql/conf.d/root.cnf | grep password '
+            '| awk "{print \$3}") database mysql -uroot -N -e '
+            '"SELECT SERVICE_STATE FROM '
+            'performance_schema.replication_connection_status"'
+        )
 
         try:
             stdout = server.execute(cmd)
@@ -75,7 +81,7 @@ def slave_is_running(running=True):
             fail("Failed to execute command %s, error: %s" %
                  (cmd, str(e)))
 
-        expected = b"ON" if running else b"OFF"
+        expected = b"ON" if running else b""
         return stdout == expected
 
     return check_slave_is_running
@@ -92,7 +98,7 @@ def backup_count_matches(count):
 
 def instance_is_active(id):
     instance = instance_info.dbaas.instances.get(id)
-    if instance.status == "ACTIVE":
+    if instance.status in CONFIG.running_status:
         return True
     else:
         assert_true(instance.status in ['PROMOTE', 'EJECT', 'BUILD', 'BACKUP'])
@@ -102,10 +108,6 @@ def instance_is_active(id):
 def create_slave():
     result = instance_info.dbaas.instances.create(
         instance_info.name + "_slave",
-        instance_info.dbaas_flavor_href,
-        {'size': 2},
-        datastore=instance_info.dbaas_datastore,
-        datastore_version=instance_info.dbaas_datastore_version,
         nics=instance_info.nics,
         replica_of=instance_info.id)
     assert_equal(200, instance_info.dbaas.last_http_code)
@@ -131,26 +133,13 @@ def validate_master(master, slaves):
     assert_true(asserted_ids.issubset(master_ids))
 
 
-@test(depends_on_groups=[configurations.CONFIGURATION_GROUP],
-      groups=[REPLICATION_GROUP, tests.INSTANCES],
+@test(depends_on_groups=[tests.DBAAS_API_CONFIGURATIONS],
+      groups=[tests.DBAAS_API_REPLICATION],
       enabled=CONFIG.swift_enabled)
 class CreateReplicationSlave(object):
-
-    @test
-    def test_replica_provisioning_with_missing_replica_source(self):
-        assert_raises(exceptions.NotFound,
-                      instance_info.dbaas.instances.create,
-                      instance_info.name + "_slave",
-                      instance_info.dbaas_flavor_href,
-                      instance_info.volume,
-                      datastore=instance_info.dbaas_datastore,
-                      datastore_version=instance_info.dbaas_datastore_version,
-                      nics=instance_info.nics,
-                      replica_of="Missing replica source")
-        assert_equal(404, instance_info.dbaas.last_http_code)
-
     @test
     def test_create_db_on_master(self):
+        """test_create_db_on_master"""
         databases = [{'name': existing_db_on_master}]
         # Ensure that the auth_token in the dbaas client is not stale
         instance_info.dbaas.authenticate()
@@ -159,51 +148,60 @@ class CreateReplicationSlave(object):
 
     @test(runs_after=['test_create_db_on_master'])
     def test_create_slave(self):
+        """test_create_slave"""
         global backup_count
         backup_count = len(
             instance_info.dbaas.instances.backups(instance_info.id))
         slave_instance.id = create_slave()
 
 
-@test(groups=[REPLICATION_GROUP, tests.INSTANCES],
+@test(groups=[tests.DBAAS_API_REPLICATION],
       enabled=CONFIG.swift_enabled,
-      depends_on=[CreateReplicationSlave])
+      depends_on_classes=[CreateReplicationSlave])
 class WaitForCreateSlaveToFinish(object):
     """Wait until the instance is created and set up as slave."""
 
     @test
-    @time_out(TIMEOUT_INSTANCE_CREATE)
+    @time_out(TIMEOUT_INSTANCE_RESTORE)
     def test_slave_created(self):
+        """Wait for replica to be created."""
         poll_until(lambda: instance_is_active(slave_instance.id))
 
 
 @test(enabled=(not CONFIG.fake_mode and CONFIG.swift_enabled),
-      depends_on=[WaitForCreateSlaveToFinish],
-      groups=[REPLICATION_GROUP, tests.INSTANCES])
+      depends_on_classes=[WaitForCreateSlaveToFinish],
+      groups=[tests.DBAAS_API_REPLICATION])
 class VerifySlave(object):
 
     def db_is_found(self, database_to_find):
 
         def find_database():
             databases = instance_info.dbaas.databases.list(slave_instance.id)
-            return (database_to_find
-                    in [d.name for d in databases])
+            return (database_to_find in [d.name for d in databases])
 
         return find_database
 
     @test
-    @time_out(20 * 60)
+    @time_out(10 * 60)
     def test_correctly_started_replication(self):
+        """test_correctly_started_replication"""
         poll_until(slave_is_running())
 
     @test(runs_after=[test_correctly_started_replication])
     @time_out(60)
     def test_backup_deleted(self):
+        """test_backup_deleted"""
         poll_until(backup_count_matches(backup_count))
 
     @test(depends_on=[test_correctly_started_replication])
     def test_slave_is_read_only(self):
-        cmd = "mysql -BNq -e \\\'select @@read_only\\\'"
+        """test_slave_is_read_only"""
+        cmd = (
+            'docker exec -e MYSQL_PWD=$(sudo cat '
+            '/var/lib/mysql/conf.d/root.cnf | grep password | '
+            'awk "{print \$3}") database mysql -uroot -NBq -e '
+            '"select @@read_only"'
+        )
         server = create_server_connection(slave_instance.id)
 
         try:
@@ -217,6 +215,7 @@ class VerifySlave(object):
 
     @test(depends_on=[test_slave_is_read_only])
     def test_create_db_on_master(self):
+        """test_create_db_on_master"""
         databases = [{'name': slave_instance.replicated_db}]
         instance_info.dbaas.databases.create(instance_info.id, databases)
         assert_equal(202, instance_info.dbaas.last_http_code)
@@ -224,38 +223,41 @@ class VerifySlave(object):
     @test(depends_on=[test_create_db_on_master])
     @time_out(5 * 60)
     def test_database_replicated_on_slave(self):
+        """test_database_replicated_on_slave"""
         poll_until(self.db_is_found(slave_instance.replicated_db))
 
     @test(runs_after=[test_database_replicated_on_slave])
     @time_out(5 * 60)
     def test_existing_db_exists_on_slave(self):
+        """test_existing_db_exists_on_slave"""
         poll_until(self.db_is_found(existing_db_on_master))
 
     @test(depends_on=[test_existing_db_exists_on_slave])
     def test_slave_user_exists(self):
+        """test_slave_user_exists"""
         assert_equal(_get_user_count(slave_instance), 1)
         assert_equal(_get_user_count(instance_info), 1)
 
 
-@test(groups=[REPLICATION_GROUP, tests.INSTANCES],
-      depends_on=[WaitForCreateSlaveToFinish],
-      runs_after=[VerifySlave],
+@test(groups=[tests.DBAAS_API_REPLICATION],
+      depends_on_classes=[VerifySlave],
       enabled=CONFIG.swift_enabled)
 class TestInstanceListing(object):
     """Test replication information in instance listing."""
 
     @test
     def test_get_slave_instance(self):
+        """test_get_slave_instance"""
         validate_slave(instance_info, slave_instance)
 
     @test
     def test_get_master_instance(self):
+        """test_get_master_instance"""
         validate_master(instance_info, [slave_instance])
 
 
-@test(groups=[REPLICATION_GROUP, tests.INSTANCES],
-      depends_on=[WaitForCreateSlaveToFinish],
-      runs_after=[TestInstanceListing],
+@test(groups=[tests.DBAAS_API_REPLICATION],
+      depends_on_classes=[TestInstanceListing],
       enabled=CONFIG.swift_enabled)
 class TestReplicationFailover(object):
     """Test replication failover functionality."""
@@ -303,14 +305,17 @@ class TestReplicationFailover(object):
     @test(depends_on=[test_promote_master, test_eject_slave,
                       test_eject_valid_master])
     def test_promote_to_replica_source(self):
+        """test_promote_to_replica_source"""
         TestReplicationFailover.promote(instance_info, slave_instance)
 
     @test(depends_on=[test_promote_to_replica_source])
     def test_promote_back_to_replica_source(self):
+        """test_promote_back_to_replica_source"""
         TestReplicationFailover.promote(slave_instance, instance_info)
 
     @test(depends_on=[test_promote_back_to_replica_source], enabled=False)
     def add_second_slave(self):
+        """add_second_slave"""
         if CONFIG.fake_mode:
             raise SkipTest("three site promote not supported in fake mode")
 
@@ -318,12 +323,13 @@ class TestReplicationFailover(object):
         self._third_slave.id = create_slave()
         poll_until(lambda: instance_is_active(self._third_slave.id))
         poll_until(slave_is_running())
-        sleep(30)
+        sleep(15)
         validate_master(instance_info, [slave_instance, self._third_slave])
         validate_slave(instance_info, self._third_slave)
 
     @test(depends_on=[add_second_slave], enabled=False)
     def test_three_site_promote(self):
+        """Promote the second slave"""
         if CONFIG.fake_mode:
             raise SkipTest("three site promote not supported in fake mode")
 
@@ -333,10 +339,11 @@ class TestReplicationFailover(object):
 
     @test(depends_on=[test_three_site_promote], enabled=False)
     def disable_master(self):
+        """Stop trove-guestagent on master"""
         if CONFIG.fake_mode:
             raise SkipTest("eject_replica_source not supported in fake mode")
 
-        cmd = "sudo service trove-guestagent stop"
+        cmd = "sudo systemctl stop guest-agent.service"
         server = create_server_connection(self._third_slave.id)
 
         try:
@@ -353,7 +360,7 @@ class TestReplicationFailover(object):
         if CONFIG.fake_mode:
             raise SkipTest("eject_replica_source not supported in fake mode")
 
-        sleep(90)
+        sleep(70)
         instance_info.dbaas.instances.eject_replica_source(self._third_slave)
         assert_equal(202, instance_info.dbaas.last_http_code)
         poll_until(lambda: instance_is_active(self._third_slave.id))
@@ -361,9 +368,8 @@ class TestReplicationFailover(object):
         validate_slave(instance_info, slave_instance)
 
 
-@test(groups=[REPLICATION_GROUP, tests.INSTANCES],
-      depends_on=[WaitForCreateSlaveToFinish],
-      runs_after=[TestReplicationFailover],
+@test(groups=[tests.DBAAS_API_REPLICATION],
+      depends_on=[TestReplicationFailover],
       enabled=CONFIG.swift_enabled)
 class DetachReplica(object):
 
@@ -376,11 +382,12 @@ class DetachReplica(object):
     @test
     @time_out(5 * 60)
     def test_detach_replica(self):
+        """test_detach_replica"""
         if CONFIG.fake_mode:
             raise SkipTest("Detach replica not supported in fake mode")
 
-        instance_info.dbaas.instances.edit(slave_instance.id,
-                                           detach_replica_source=True)
+        instance_info.dbaas.instances.update(slave_instance.id,
+                                             detach_replica_source=True)
         assert_equal(202, instance_info.dbaas.last_http_code)
 
         poll_until(slave_is_running(False))
@@ -388,12 +395,18 @@ class DetachReplica(object):
     @test(depends_on=[test_detach_replica])
     @time_out(5 * 60)
     def test_slave_is_not_read_only(self):
+        """test_slave_is_not_read_only"""
         if CONFIG.fake_mode:
             raise SkipTest("Test not_read_only not supported in fake mode")
 
         # wait until replica is no longer read only
         def check_not_read_only():
-            cmd = "mysql -BNq -e \\\'select @@read_only\\\'"
+            cmd = (
+                'docker exec -e MYSQL_PWD=$(sudo cat '
+                '/var/lib/mysql/conf.d/root.cnf | grep password | '
+                'awk "{print \$3}") database mysql -uroot -NBq -e '
+                '"select @@read_only"'
+            )
             server = create_server_connection(slave_instance.id)
 
             try:
@@ -407,15 +420,15 @@ class DetachReplica(object):
         poll_until(check_not_read_only)
 
 
-@test(groups=[REPLICATION_GROUP, tests.INSTANCES],
-      depends_on=[WaitForCreateSlaveToFinish],
-      runs_after=[DetachReplica],
+@test(groups=[tests.DBAAS_API_REPLICATION],
+      depends_on=[DetachReplica],
       enabled=CONFIG.swift_enabled)
 class DeleteSlaveInstance(object):
 
     @test
     @time_out(TIMEOUT_INSTANCE_DELETE)
     def test_delete_slave_instance(self):
+        """test_delete_slave_instance"""
         instance_info.dbaas.instances.delete(slave_instance.id)
         assert_equal(202, instance_info.dbaas.last_http_code)
 
