@@ -11,44 +11,48 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import json
 import os
+
 from tempfile import NamedTemporaryFile
 from unittest import mock
-from unittest.mock import call
 from unittest.mock import MagicMock
 from unittest.mock import Mock
-from unittest.mock import patch
 from unittest.mock import PropertyMock
+from unittest.mock import call
+from unittest.mock import patch
+
+import cinderclient.v3.client as cinderclient
+import neutronclient.v2_0.client as neutronclient
+import novaclient.v2.flavors
+import novaclient.v2.servers
 
 from cinderclient import exceptions as cinder_exceptions
 from cinderclient.v3 import volumes as cinderclient_volumes
-import cinderclient.v3.client as cinderclient
-import neutronclient.v2_0.client as neutronclient
 from novaclient import exceptions as nova_exceptions
-import novaclient.v2.flavors
-import novaclient.v2.servers
 from oslo_config import cfg
 from swiftclient.client import ClientException
 from testtools.matchers import Equals
 from testtools.matchers import Is
 
+import trove.backup.models
+import trove.common.context
+import trove.common.template as template
+import trove.db.models
+import trove.guestagent.api
+
 from trove import rpc
 from trove.backup import models as backup_models
 from trove.backup import state
-import trove.backup.models
+from trove.common import exception
 from trove.common import timeutils
 from trove.common import utils
-import trove.common.context
-from trove.common import exception
 from trove.common.exception import GuestError
 from trove.common.exception import PollTimeOut
 from trove.common.exception import TroveError
-import trove.common.template as template
 from trove.datastore import models as datastore_models
-import trove.db.models
 from trove.extensions.common import models as common_models
 from trove.extensions.mysql import models as mysql_models
-import trove.guestagent.api
 from trove.instance.models import BaseInstance
 from trove.instance.models import DBInstance
 from trove.instance.models import InstanceServiceStatus
@@ -64,6 +68,7 @@ VOLUME_ID = 'volume-id-1'
 
 
 class FakeOptGroup(object):
+
     def __init__(self, tcp_ports=['3306', '3301-3307'],
                  udp_ports=[], icmp=False):
         self.tcp_ports = tcp_ports
@@ -72,6 +77,7 @@ class FakeOptGroup(object):
 
 
 class fake_Server(object):
+
     def __init__(self):
         self.id = None
         self.name = None
@@ -108,6 +114,7 @@ class fake_ServerManager(object):
 
 
 class fake_nova_client(object):
+
     def __init__(self):
         self.servers = fake_ServerManager()
 
@@ -214,7 +221,7 @@ class BaseFreshInstanceTasksTest(trove_testtools.TestCase):
         self.freshinstancetasks = taskmanager_models.FreshInstanceTasks(
             None, MagicMock(), None, None)
         self.freshinstancetasks.context = trove.common.context.TroveContext(
-            user='test_user')
+            user_id='test_user')
 
     def tearDown(self):
         super(BaseFreshInstanceTasksTest, self).tearDown()
@@ -246,11 +253,18 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
 
         self.assertEqual('fake_keypair', server.key_name)
 
+    def test_create_instance_prepare_cloud_config(self):
+        files = {"/tmp/fake_file": "fake text"}
+        user_data = self.freshinstancetasks.prepare_cloud_config(files)
+        self.assertTrue(user_data.startswith('#cloud-config'))
+
     @patch.object(DBInstance, 'get_by')
     def test_create_instance_guestconfig(self, patch_get_by):
         cfg.CONF.set_override('guest_config', self.guestconfig)
         cfg.CONF.set_override('guest_info', 'guest_info.conf')
         cfg.CONF.set_override('injected_config_location', '/etc/trove/conf.d')
+        cfg.CONF.set_override('docker_insecure_registries',
+                              '127.0.0.1:4000,127.0.0.1:5000')
 
         # execute
         files = self.freshinstancetasks.get_injected_files("test", 'test')
@@ -262,6 +276,9 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         self.assertEqual(
             self.guestconfig_content,
             files['/etc/trove/conf.d/trove-guestagent.conf'])
+        deamon_json = json.loads(files.get('/etc/docker/daemon.json'))
+        self.assertIn('127.0.0.1:4000', deamon_json.get('insecure-registries'))
+        self.assertIn('127.0.0.1:5000', deamon_json.get('insecure-registries'))
 
     @patch.object(DBInstance, 'get_by')
     def test_create_instance_guestconfig_compat(self, patch_get_by):
@@ -279,6 +296,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         self.assertEqual(
             self.guestconfig_content,
             files['/etc/trove-guestagent.conf'])
+        self.assertFalse(files.get('/etc/docker/daemon.json'))
 
     def test_create_instance_with_az_kwarg(self):
         # execute
@@ -310,6 +328,10 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
     def test_servers_create_block_device_mapping_v2(self,
                                                     mock_hostname,
                                                     mock_name):
+        # This testcase is to test create_server with config_drive=True.
+        # We need set use_nova_server_config_drive True explicitly
+        # because use_nova_server_config_drive becomes False since Yoga
+        cfg.CONF.set_override('use_nova_server_config_drive', True)
         self.freshinstancetasks.prepare_userdata = Mock(return_value=None)
         mock_nova_client = self.freshinstancetasks.nova_client = Mock()
         mock_servers_create = mock_nova_client.servers.create
@@ -331,6 +353,42 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
             meta=meta,
         )
 
+    @patch.object(taskmanager_models.FreshInstanceTasks, 'hostname',
+                  new_callable=PropertyMock,
+                  return_value='fake-hostname')
+    @patch.object(taskmanager_models.FreshInstanceTasks, 'name',
+                  new_callable=PropertyMock,
+                  return_value='fake-name')
+    def test_servers_create_block_device_mapping_v2_since_yoga(
+        self, mock_hostname, mock_name):
+
+        # This testcase is for testing with config_drive=False.
+        self.freshinstancetasks.prepare_userdata = Mock(
+            return_value="#cloud-config\nwrite_files:\n")
+        mock_nova_client = self.freshinstancetasks.nova_client = Mock()
+        mock_servers_create = mock_nova_client.servers.create
+        self.freshinstancetasks._create_server('fake-flavor', 'fake-image',
+                                               'mysql', None, None, None)
+        meta = {'trove_project_id': self.freshinstancetasks.tenant_id,
+                'trove_user_id': 'test_user',
+                'trove_instance_id': self.freshinstancetasks.id}
+
+        userdata = self.freshinstancetasks.prepare_userdata('mysql')
+        userdata = userdata + \
+            self.freshinstancetasks.prepare_cloud_config({})
+        mock_servers_create.assert_called_with(
+            'fake-name', 'fake-image',
+            'fake-flavor', files={},
+            userdata=userdata,
+            block_device_mapping_v2=None,
+            availability_zone=None,
+            nics=None,
+            config_drive=False,
+            scheduler_hints=None,
+            key_name=None,
+            meta=meta,
+        )
+
     @patch.object(InstanceServiceStatus, 'find_by',
                   return_value=fake_InstanceServiceStatus.find_by())
     @patch.object(DBInstance, 'find_by',
@@ -599,6 +657,7 @@ class ResizeVolumeTest(trove_testtools.TestCase):
                                                             self.new_vol_size)
 
         class FakeGroup(object):
+
             def __init__(self):
                 self.mount_point = 'var/lib/mysql'
                 self.device_path = '/dev/vdb'
@@ -1105,6 +1164,7 @@ class BackupTasksTest(trove_testtools.TestCase):
 
 
 class NotifyMixinTest(trove_testtools.TestCase):
+
     def test_get_service_id(self):
         id_map = {
             'mysql': '123',
@@ -1135,16 +1195,16 @@ class RootReportTest(trove_testtools.TestCase):
 
     def test_report_root_first_time(self):
         context = Mock()
-        context.user = utils.generate_uuid()
+        context.user_id = utils.generate_uuid()
         report = mysql_models.RootHistory.create(
             context, utils.generate_uuid())
         self.assertIsNotNone(report)
 
     def test_report_root_double_create(self):
         context = Mock()
-        context.user = utils.generate_uuid()
+        context.user_id = utils.generate_uuid()
         id = utils.generate_uuid()
-        history = mysql_models.RootHistory(id, context.user).save()
+        history = mysql_models.RootHistory(id, context.user_id).save()
         with patch.object(mysql_models.RootHistory, 'load',
                           Mock(return_value=history)):
             report = mysql_models.RootHistory.create(context, id)
@@ -1159,7 +1219,7 @@ class ClusterRootTest(trove_testtools.TestCase):
     @patch.object(common_models.Root, "create")
     def test_cluster_root_create(self, root_create, root_history_create):
         context = Mock()
-        context.user = utils.generate_uuid()
+        context.user_id = utils.generate_uuid()
         id = utils.generate_uuid()
         password = "rootpassword"
         cluster_instances = [utils.generate_uuid(), utils.generate_uuid()]
